from dataclasses import dataclass
from typing import List, Optional

from sinch.core.models.base_model import SinchBaseModel
from sinch.domains.numbers.models.active import ActiveNumber


@dataclass
class ListActiveNumbersResponse(SinchBaseModel):
    active_numbers: List[ActiveNumber]
    next_page_token: Optional[str] = None


@dataclass
class UpdateNumberConfigurationResponse(ActiveNumber):
    pass


@dataclass
class GetNumberConfigurationResponse(ActiveNumber):
    pass


@dataclass
class ReleaseNumberFromProjectResponse(ActiveNumber):
    pass



from sinch.core.models.http_response import HTTPResponse
from sinch.domains.conversation.endpoints.conversation_endpoint import ConversationEndpoint
from sinch.core.enums import HTTPAuthentication, HTTPMethods
from sinch.domains.conversation.models.templates.responses import UpdateConversationTemplateResponse
from sinch.domains.conversation.models.templates.requests import UpdateConversationTemplateRequest


class UpdateTemplateEndpoint(ConversationEndpoint):
    ENDPOINT_URL = "{origin}/v1/projects/{project_id}/templates/{template_id}"
    HTTP_METHOD = HTTPMethods.PATCH.value
    HTTP_AUTHENTICATION = HTTPAuthentication.OAUTH.value

    def __init__(self, project_id: str, request_data: UpdateConversationTemplateRequest):
        super(UpdateTemplateEndpoint, self).__init__(project_id, request_data)
        self.request_data = request_data
        self.project_id = project_id

    def build_url(self, sinch):
        return self.ENDPOINT_URL.format(
            origin=sinch.configuration.templates_origin,
            project_id=self.project_id,
            template_id=self.request_data.template_id
        )

    def request_body(self):
        self.request_data.template_id = None
        return self.request_data.as_json()

    def handle_response(self, response: HTTPResponse) -> UpdateConversationTemplateResponse:
        super(UpdateTemplateEndpoint, self).handle_response(response)
        return UpdateConversationTemplateResponse(
            id=response.body["id"],
            description=response.body["description"],
            default_translation=response.body["default_translation"],
            create_time=response.body["create_time"],
            translations=response.body["translations"],
            update_time=response.body["update_time"],
            channel=response.body["channel"]
        )



from dataclasses import dataclass
from sinch.core.models.base_model import SinchBaseModel


@dataclass
class RegisterConversationOptInResponse(SinchBaseModel):
    request_id: str
    opt_in: dict


@dataclass
class RegisterConversationOptOutResponse(SinchBaseModel):
    request_id: str
    opt_out: dict



from dataclasses import dataclass
from typing import Optional
from sinch.core.models.base_model import SinchRequestBaseModel


@dataclass
class ListConversationMessagesRequest(SinchRequestBaseModel):
    conversation_id: Optional[str]
    contact_id: Optional[str]
    app_id: Optional[str]
    page_size: Optional[int]
    page_token: Optional[str]
    view: Optional[str]
    messages_source: Optional[str]
    only_recipient_originated: Optional[bool]


@dataclass
class GetConversationMessageRequest(SinchRequestBaseModel):
    message_id: str
    messages_source: str


@dataclass
class DeleteConversationMessageRequest(SinchRequestBaseModel):
    message_id: str
    messages_source: str


@dataclass
class SendConversationMessageRequest(SinchRequestBaseModel):
    app_id: str
    recipient: dict
    message: dict
    callback_url: str
    processing_strategy: Optional[str]
    channel_priority_order: list
    channel_properties: dict
    message_metadata: str
    conversation_metadata: dict
    queue: str
    ttl: str
    processing_strategy: str



from dataclasses import dataclass
from sinch.core.models.base_model import SinchBaseModel


@dataclass
class OAuthToken(SinchBaseModel):
    access_token: str
    expires_in: int
    scope: str
    token_type: str



import json
from dataclasses import asdict, dataclass


@dataclass
class SinchBaseModel:
    def as_dict(self):
        return asdict(self)

    def as_json(self):
        return json.dumps(self.as_dict())


@dataclass
class SinchRequestBaseModel(SinchBaseModel):
    def as_dict(self):
        return {k: v for k, v in asdict(self).items() if v is not None}



import json
from sinch.core.models.http_response import HTTPResponse
from sinch.domains.conversation.endpoints.conversation_endpoint import ConversationEndpoint
from sinch.core.enums import HTTPAuthentication, HTTPMethods
from sinch.domains.conversation.models.contact.responses import MergeConversationContactsResponse


class MergeConversationContactsEndpoint(ConversationEndpoint):
    ENDPOINT_URL = "{origin}/v1/projects/{project_id}/contacts/{destination_id}:merge"
    HTTP_METHOD = HTTPMethods.POST.value
    HTTP_AUTHENTICATION = HTTPAuthentication.OAUTH.value

    def __init__(self, project_id, request_data):
        super(MergeConversationContactsEndpoint, self).__init__(project_id, request_data)
        self.project_id = project_id
        self.request_data = request_data

    def build_url(self, sinch):
        return self.ENDPOINT_URL.format(
            origin=sinch.configuration.conversation_origin,
            project_id=self.project_id,
            destination_id=self.request_data.destination_id
        )

    def request_body(self):
        return json.dumps({
            "source_id": self.request_data.source_id
        })

    def handle_response(self, response: HTTPResponse) -> MergeConversationContactsResponse:
        super(MergeConversationContactsEndpoint, self).handle_response(response)
        return MergeConversationContactsResponse(
            **response.body
        )



from typing import List, Literal, Union
from sinch.domains.voice.endpoints.callouts.callout import CalloutEndpoint
from sinch.domains.voice.endpoints.calls.get_call import GetCallEndpoint
from sinch.domains.voice.endpoints.calls.update_call import UpdateCallEndpoint
from sinch.domains.voice.endpoints.calls.manage_call import ManageCallEndpoint

from sinch.domains.voice.endpoints.applications.get_numbers import GetVoiceNumbersEndpoint
from sinch.domains.voice.endpoints.applications.query_number import QueryVoiceNumberEndpoint
from sinch.domains.voice.endpoints.applications.get_callback_urls import GetVoiceCallbacksEndpoint
from sinch.domains.voice.endpoints.applications.unassign_number import UnAssignVoiceNumberEndpoint
from sinch.domains.voice.endpoints.applications.assign_numbers import AssignVoiceNumbersEndpoint
from sinch.domains.voice.endpoints.applications.update_callbacks import UpdateVoiceCallbacksEndpoint

from sinch.domains.voice.endpoints.conferences.kick_participant import KickParticipantConferenceEndpoint
from sinch.domains.voice.endpoints.conferences.kick_all_participants import KickAllConferenceEndpoint
from sinch.domains.voice.endpoints.conferences.manage_participant import ManageParticipantConferenceEndpoint
from sinch.domains.voice.endpoints.conferences.get_conference import GetConferenceEndpoint

from sinch.domains.voice.enums import CalloutMethod
from sinch.domains.voice.models.callouts.responses import VoiceCalloutResponse
from sinch.domains.voice.models.callouts.requests import (
    ConferenceVoiceCalloutRequest,
    TextToSpeechVoiceCalloutRequest,
    CustomVoiceCalloutRequest,
    Destination,
    ConferenceDTMFOptions
)
from sinch.domains.voice.models.calls.requests import (
    GetVoiceCallRequest,
    UpdateVoiceCallRequest,
    ManageVoiceCallRequest
)
from sinch.domains.voice.models.calls.responses import (
    GetVoiceCallResponse,
    UpdateVoiceCallResponse,
    ManageVoiceCallResponse
)
from sinch.domains.voice.models.conferences.requests import (
    GetVoiceConferenceRequest,
    KickAllVoiceConferenceRequest,
    KickParticipantVoiceConferenceRequest,
    ManageParticipantVoiceConferenceRequest
)
from sinch.domains.voice.models.conferences.responses import (
    GetVoiceConferenceResponse,
    KickAllVoiceConferenceResponse,
    ManageParticipantVoiceConferenceResponse,
    KickParticipantVoiceConferenceResponse
)
from sinch.domains.voice.models.applications.requests import (
    AssignNumbersVoiceApplicationRequest,
    UnassignNumbersVoiceApplicationRequest,
    QueryNumberVoiceApplicationRequest,
    UpdateCallbackUrlsVoiceApplicationRequest,
    GetCallbackUrlsVoiceApplicationRequest
)
from sinch.domains.voice.models.applications.responses import (
    GetNumbersVoiceApplicationResponse,
    AssignNumbersVoiceApplicationResponse,
    UnassignNumbersVoiceApplicationResponse,
    GetCallbackUrlsVoiceApplicationResponse,
    QueryNumberVoiceApplicationResponse
)
from sinch.domains.voice.models.svaml.actions.actions import Action
from sinch.domains.voice.models.svaml.instructions.instructions import Instruction


class Callouts:
    def __init__(self, sinch):
        self._sinch = sinch

    def text_to_speech(
        self,
        destination: Destination,
        cli: str = None,
        dtmf: str = None,
        domain: Literal["pstn", "mxp"] = None,
        custom: str = None,
        locale: str = None,
        text: str = None,
        prompts: str = None,
        enable_ace: bool = None,
        enable_dice: bool = None,
        enable_pie: bool = None
    ) -> VoiceCalloutResponse:
        return self._sinch.configuration.transport.request(
            CalloutEndpoint(
                callout_method=CalloutMethod.TEXT_TO_SPEECH.value,
                request_data=TextToSpeechVoiceCalloutRequest(
                    destination=destination,
                    cli=cli,
                    dtmf=dtmf,
                    domain=domain,
                    custom=custom,
                    locale=locale,
                    text=text,
                    prompts=prompts,
                    enableAce=enable_ace,
                    enableDice=enable_dice,
                    enablePie=enable_pie
                )
            )
        )

    def conference(
        self,
        destination: Destination,
        conference_id: str,
        cli: str = None,
        conference_dtmf_options: ConferenceDTMFOptions = None,
        dtmf: str = None,
        conference: str = None,
        max_duration: int = None,
        enable_ace: bool = None,
        enable_dice: bool = None,
        enable_pie: bool = None,
        locale: str = None,
        greeting: str = None,
        moh_class: str = None,
        custom: str = None,
        domain: Literal["pstn", "mxp"] = None,
    ) -> VoiceCalloutResponse:
        return self._sinch.configuration.transport.request(
            CalloutEndpoint(
                callout_method=CalloutMethod.CONFERENCE.value,
                request_data=ConferenceVoiceCalloutRequest(
                    destination=destination,
                    conferenceId=conference_id,
                    cli=cli,
                    conferenceDtmfOptions=conference_dtmf_options,
                    dtmf=dtmf,
                    conference=conference,
                    maxDuration=max_duration,
                    enableAce=enable_ace,
                    enableDice=enable_dice,
                    enablePie=enable_pie,
                    locale=locale,
                    greeting=greeting,
                    mohClass=moh_class,
                    custom=custom,
                    domain=domain
                )
            )
        )

    def custom(
        self,
        cli: str = None,
        destination: Destination = None,
        dtmf: str = None,
        custom: str = None,
        max_duration: int = None,
        ice: str = None,
        ace: str = None,
        pie: str = None
    ) -> VoiceCalloutResponse:
        return self._sinch.configuration.transport.request(
            CalloutEndpoint(
                callout_method=CalloutMethod.CUSTOM.value,
                request_data=CustomVoiceCalloutRequest(
                    cli=cli,
                    destination=destination,
                    dtmf=dtmf,
                    custom=custom,
                    maxDuration=max_duration,
                    ice=ice,
                    ace=ace,
                    pie=pie
                )
            )
        )


class Calls:
    def __init__(self, sinch):
        self._sinch = sinch

    def get(self, call_id) -> GetVoiceCallResponse:
        return self._sinch.configuration.transport.request(
            GetCallEndpoint(
                request_data=GetVoiceCallRequest(
                    call_id=call_id
                )
            )
        )

    def update(
        self,
        call_id: str,
        instructions: Union[list, List[Instruction]],
        action: Union[dict, Action]
    ) -> UpdateVoiceCallResponse:
        return self._sinch.configuration.transport.request(
            UpdateCallEndpoint(
                request_data=UpdateVoiceCallRequest(
                    call_id=call_id,
                    instructions=instructions,
                    action=action
                )
            )
        )

    def manage_with_call_leg(
        self,
        call_id: str,
        call_leg: str,
        instructions: Union[list, List[Instruction]],
        action: Union[dict, Action]
    ) -> ManageVoiceCallResponse:
        return self._sinch.configuration.transport.request(
            ManageCallEndpoint(
                request_data=ManageVoiceCallRequest(
                    call_id=call_id,
                    call_leg=call_leg,
                    instructions=instructions,
                    action=action
                )
            )
        )


class Conferences:
    def __init__(self, sinch):
        self._sinch = sinch

    def call(
        self,
        destination: Destination,
        conference_id: str,
        cli: str = None,
        conference_dtmf_options: ConferenceDTMFOptions = None,
        dtmf: str = None,
        conference: str = None,
        max_duration: int = None,
        enable_ace: bool = None,
        enable_dice: bool = None,
        enable_pie: bool = None,
        locale: str = None,
        greeting: str = None,
        moh_class: str = None,
        custom: str = None,
        domain: Literal["pstn", "mxp"] = None,
    ) -> VoiceCalloutResponse:
        return self._sinch.voice.callouts.conference(
            destination=destination,
            conference_id=conference_id,
            cli=cli,
            conference_dtmf_options=conference_dtmf_options,
            dtmf=dtmf,
            conference=conference,
            max_duration=max_duration,
            enable_ace=enable_ace,
            enable_dice=enable_dice,
            enable_pie=enable_pie,
            locale=locale,
            greeting=greeting,
            moh_class=moh_class,
            custom=custom,
            domain=domain
        )

    def get(self, conference_id: str) -> GetVoiceConferenceResponse:
        return self._sinch.configuration.transport.request(
            GetConferenceEndpoint(
                request_data=GetVoiceConferenceRequest(
                    conference_id=conference_id
                )
            )
        )

    def kick_all(self, conference_id: str) -> KickAllVoiceConferenceResponse:
        return self._sinch.configuration.transport.request(
            KickAllConferenceEndpoint(
                request_data=KickAllVoiceConferenceRequest(
                    conference_id=conference_id
                )
            )
        )

    def kick_participant(
        self,
        call_id: str,
        conference_id: str,
    ) -> KickParticipantVoiceConferenceResponse:
        return self._sinch.configuration.transport.request(
            KickParticipantConferenceEndpoint(
                request_data=KickParticipantVoiceConferenceRequest(
                    call_id=call_id,
                    conference_id=conference_id
                )
            )
        )

    def manage_participant(
        self,
        call_id: str,
        conference_id: str,
        command: str,
        moh: str = None
    ) -> ManageParticipantVoiceConferenceResponse:
        return self._sinch.configuration.transport.request(
            ManageParticipantConferenceEndpoint(
                request_data=ManageParticipantVoiceConferenceRequest(
                    call_id=call_id,
                    conference_id=conference_id,
                    command=command,
                    moh=moh
                )
            )
        )


class Applications:
    def __init__(self, sinch):
        self._sinch = sinch

    def get_numbers(self) -> GetNumbersVoiceApplicationResponse:
        return self._sinch.configuration.transport.request(
            GetVoiceNumbersEndpoint()
        )

    def assign_numbers(
        self,
        numbers: List[str],
        application_key: str = None,
        capability: str = None
    ) -> AssignNumbersVoiceApplicationResponse:
        return self._sinch.configuration.transport.request(
            AssignVoiceNumbersEndpoint(
                request_data=AssignNumbersVoiceApplicationRequest(
                    numbers=numbers,
                    application_key=application_key,
                    capability=capability
                )
            )
        )

    def unassign_number(
        self,
        number: str,
        application_key: str = None,
        capability: str = None

    ) -> UnassignNumbersVoiceApplicationResponse:
        return self._sinch.configuration.transport.request(
            UnAssignVoiceNumberEndpoint(
                request_data=UnassignNumbersVoiceApplicationRequest(
                    number=number,
                    application_key=application_key,
                    capability=capability
                )
            )
        )

    def get_callback_urls(
        self,
        application_key: str
    ) -> GetCallbackUrlsVoiceApplicationResponse:
        return self._sinch.configuration.transport.request(
            GetVoiceCallbacksEndpoint(
                request_data=GetCallbackUrlsVoiceApplicationRequest(
                    application_key=application_key
                )
            )
        )

    def update_callback_urls(
        self,
        application_key: str,
        primary: str = None,
        fallback: str = None
    ):
        return self._sinch.configuration.transport.request(
            UpdateVoiceCallbacksEndpoint(
                request_data=UpdateCallbackUrlsVoiceApplicationRequest(
                    application_key=application_key,
                    primary=primary,
                    fallback=fallback
                )
            )
        )

    def query_number(self, number) -> QueryNumberVoiceApplicationResponse:
        return self._sinch.configuration.transport.request(
            QueryVoiceNumberEndpoint(
                request_data=QueryNumberVoiceApplicationRequest(
                    number=number
                )
            )
        )


class VoiceBase:
    """
    Documentation for the Voice API: https://developers.sinch.com/docs/voice/
    """
    def __init__(self, sinch):
        self._sinch = sinch


class Voice(VoiceBase):
    """
    Synchronous version of the Voice Domain
    """
    __doc__ += VoiceBase.__doc__

    def __init__(self, sinch):
        super().__init__(sinch)
        self.callouts = Callouts(self._sinch)
        self.calls = Calls(self._sinch)
        self.conferences = Conferences(self._sinch)
        self.applications = Applications(self._sinch)


class VoiceAsync(VoiceBase):
    """
    Asynchronous version of the Voice Domain
    """
    __doc__ += VoiceBase.__doc__

    def __init__(self, sinch):
        super().__init__(sinch)
        self.callouts = Callouts(self._sinch)
        self.calls = Calls(self._sinch)
        self.conferences = Conferences(self._sinch)
        self.applications = Applications(self._sinch)



from dataclasses import dataclass
from sinch.core.models.base_model import SinchBaseModel


@dataclass
class ListSMSDeliveryReportsResponse(SinchBaseModel):
    page: str
    page_size: str
    count: str
    delivery_reports: list


@dataclass
class GetSMSDeliveryReportForBatchResponse(SinchBaseModel):
    type: str
    batch_id: str
    total_message_count: str
    statuses: list
    client_reference: str


@dataclass
class GetSMSDeliveryReportForNumberResponse(SinchBaseModel):
    at: str
    batch_id: str
    code: int
    recipient: str
    status: str
    applied_originator: str
    client_reference: str
    number_of_message_parts: str
    operator: str
    operator_status_at: str
    type: str



from sinch.core.models.http_response import HTTPResponse
from sinch.domains.verification.endpoints.verification_endpoint import VerificationEndpoint
from sinch.core.enums import HTTPAuthentication, HTTPMethods
from sinch.domains.verification.models.requests import GetVerificationStatusByIdRequest
from sinch.domains.verification.models.responses import GetVerificationStatusByIdResponse


class GetVerificationStatusByIdEndpoint(VerificationEndpoint):
    ENDPOINT_URL = "{origin}/verification/v1/verifications/id/{id}"
    HTTP_METHOD = HTTPMethods.GET.value
    HTTP_AUTHENTICATION = HTTPAuthentication.SIGNED.value

    def __init__(self, request_data: GetVerificationStatusByIdRequest):
        self.request_data = request_data

    def build_url(self, sinch):
        return self.ENDPOINT_URL.format(
            origin=sinch.configuration.verification_origin,
            id=self.request_data.id
        )

    def handle_response(self, response: HTTPResponse) -> GetVerificationStatusByIdResponse:
        super().handle_response(response)
        return GetVerificationStatusByIdResponse(
            id=response.body.get("id"),
            method=response.body.get("method"),
            status=response.body.get("status"),
            price=response.body.get("price"),
            identity=response.body.get("identity"),
            country_id=response.body.get("country_id"),
            verification_timestamp=response.body.get("verification_timestamp"),
            reference=response.body.get("reference"),
            reason=response.body.get("reason"),
            call_complete=response.body.get("call_complete")
        )



from .actions import (
    AnsweringMachineDetection, CallHeader, HangupAction,
    ContinueAction, ConnectPstnAction, ConnectMxpAction,
    Option, MenuOption, ConnectSipAction, ConnectConfAction,
    RunMenuAction, ParkAction
)

__all__ = [
    "AnsweringMachineDetection", "CallHeader", "HangupAction",
    "ContinueAction", "ConnectPstnAction", "ConnectMxpAction",
    "Option", "MenuOption", "ConnectSipAction", "ConnectConfAction",
    "RunMenuAction", "ParkAction"
]



from sinch.core.models.http_response import HTTPResponse
from sinch.domains.conversation.endpoints.conversation_endpoint import ConversationEndpoint
from sinch.core.enums import HTTPAuthentication, HTTPMethods
from sinch.domains.conversation.models.templates.responses import ListConversationTemplatesResponse
from sinch.domains.conversation.models.templates import ConversationTemplate


class ListTemplatesEndpoint(ConversationEndpoint):
    ENDPOINT_URL = "{origin}/v1/projects/{project_id}/templates"
    HTTP_METHOD = HTTPMethods.GET.value
    HTTP_AUTHENTICATION = HTTPAuthentication.OAUTH.value

    def __init__(self, project_id: str, request_data=None):
        super(ListTemplatesEndpoint, self).__init__(project_id, request_data)
        self.request_data = request_data
        self.project_id = project_id

    def build_url(self, sinch):
        return self.ENDPOINT_URL.format(
            origin=sinch.configuration.templates_origin,
            project_id=self.project_id
        )

    def handle_response(self, response: HTTPResponse) -> ListConversationTemplatesResponse:
        super(ListTemplatesEndpoint, self).handle_response(response)
        return ListConversationTemplatesResponse(
            templates=[
                ConversationTemplate(
                    id=template["id"],
                    description=template["description"],
                    default_translation=template["default_translation"],
                    create_time=template["create_time"],
                    translations=template["translations"],
                    update_time=template["update_time"],
                    channel=template["channel"]
                ) for template in response.body["templates"]
            ]
        )



from sinch.core.models.http_response import HTTPResponse
from sinch.domains.conversation.endpoints.conversation_endpoint import ConversationEndpoint
from sinch.domains.conversation.models.conversation.responses import SinchListConversationsResponse
from sinch.domains.conversation.models.conversation.requests import ListConversationsRequest
from sinch.domains.conversation.models.conversation import Conversation
from sinch.core.enums import HTTPAuthentication, HTTPMethods


class ListConversationsEndpoint(ConversationEndpoint):
    ENDPOINT_URL = "{origin}/v1/projects/{project_id}/conversations"
    HTTP_METHOD = HTTPMethods.GET.value
    HTTP_AUTHENTICATION = HTTPAuthentication.OAUTH.value

    def __init__(self, project_id: str, request_data: ListConversationsRequest):
        super(ListConversationsEndpoint, self).__init__(project_id, request_data)
        self.request_data = request_data
        self.project_id = project_id

    def build_url(self, sinch):
        return self.ENDPOINT_URL.format(
            origin=sinch.configuration.conversation_origin,
            project_id=self.project_id
        )

    def build_query_params(self):
        query_params = {}
        if self.request_data.app_id:
            query_params["app_id"] = self.request_data.app_id

        if self.request_data.contact_id:
            query_params["contact_id"] = self.request_data.contact_id

        if self.request_data.page_size:
            query_params["page_size"] = self.request_data.page_size

        if self.request_data.page_token:
            query_params["page_token"] = self.request_data.page_token

        return query_params

    def handle_response(self, response: HTTPResponse) -> SinchListConversationsResponse:
        super(ListConversationsEndpoint, self).handle_response(response)
        return SinchListConversationsResponse(
            conversations=[
                Conversation(
                    id=conversation["id"],
                    app_id=conversation["app_id"],
                    contact_id=conversation["contact_id"],
                    last_received=conversation["last_received"],
                    active_channel=conversation["active_channel"],
                    active=conversation["active"],
                    metadata=conversation["metadata"],
                    metadata_json=conversation["metadata_json"]
                ) for conversation in response.body["conversations"]
            ],
            next_page_token=response.body["next_page_token"],
            total_size=response.body["total_size"]
        )



from sinch.core.models.http_response import HTTPResponse
from sinch.core.endpoint import HTTPEndpoint
from sinch.domains.verification.exceptions import VerificationException


class VerificationEndpoint(HTTPEndpoint):
    def handle_response(self, response: HTTPResponse):
        if response.status_code >= 400:
            raise VerificationException(
                message=response.body["message"],
                response=response,
                is_from_server=True
            )



from sinch.core.models.http_response import HTTPResponse
from sinch.domains.conversation.endpoints.conversation_endpoint import ConversationEndpoint
from sinch.core.enums import HTTPAuthentication, HTTPMethods
from sinch.domains.conversation.models.app.responses import DeleteConversationAppResponse
from sinch.domains.conversation.models.app.requests import DeleteConversationAppRequest


class DeleteConversationAppEndpoint(ConversationEndpoint):
    ENDPOINT_URL = "{origin}/v1/projects/{project_id}/apps/{app_id}"
    HTTP_METHOD = HTTPMethods.DELETE.value
    HTTP_AUTHENTICATION = HTTPAuthentication.OAUTH.value

    def __init__(self, project_id: str, request_data: DeleteConversationAppRequest):
        super(DeleteConversationAppEndpoint, self).__init__(project_id, request_data)
        self.project_id = project_id
        self.request_data = request_data

    def build_url(self, sinch):
        return self.ENDPOINT_URL.format(
            origin=sinch.configuration.conversation_origin,
            project_id=self.project_id,
            app_id=self.request_data.app_id
        )

    def handle_response(self, response: HTTPResponse) -> DeleteConversationAppResponse:
        super(DeleteConversationAppEndpoint, self).handle_response(response)
        return DeleteConversationAppResponse()



from dataclasses import dataclass
from typing import Optional
from sinch.core.models.base_model import SinchRequestBaseModel
from sinch.domains.conversation.models import (
    SinchConversationRetentionPolicy
)
from sinch.domains.conversation.enums import (
    ConversationMetadataReportView,
    ConversationProcessingMode
)


@dataclass
class CreateConversationAppRequest(SinchRequestBaseModel):
    display_name: str
    channel_credentials: Optional[list]
    processing_mode: Optional[ConversationProcessingMode]
    conversation_metadata_report_view: Optional[ConversationMetadataReportView]
    retention_policy: Optional[SinchConversationRetentionPolicy]
    dispatch_retention_policy: Optional[SinchConversationRetentionPolicy]


@dataclass
class DeleteConversationAppRequest(SinchRequestBaseModel):
    app_id: str


@dataclass
class GetConversationAppRequest(SinchRequestBaseModel):
    app_id: str


@dataclass
class UpdateConversationAppRequest(CreateConversationAppRequest):
    app_id: str
    update_mask: list



from logging import Logger
from abc import ABC, abstractmethod
from sinch.core.clients.sinch_client_configuration import Configuration
from sinch.domains.authentication import AuthenticationBase
from sinch.domains.numbers import NumbersBase
from sinch.domains.conversation import ConversationBase
from sinch.domains.sms import SMSBase
from sinch.domains.voice import VoiceBase


class SinchClientBase(ABC):
    """
    Sinch abstract base class for concrete Sinch Client implementations.
    By default, this SDK provides two implementations - sync and async.
    Feel free to utilize any of them for you custom implementation.
    """
    configuration = Configuration
    authentication = AuthenticationBase
    numbers = NumbersBase
    conversation = ConversationBase
    sms = SMSBase
    voice = VoiceBase

    @abstractmethod
    def __init__(
        self,
        key_id: str = None,
        key_secret: str = None,
        project_id: str = None,
        logger_name: str = None,
        logger: Logger = None,
        application_key: str = None,
        application_secret: str = None,
        service_plan_id: str = None,
        sms_api_token: str = None
    ):
        pass

    def __repr__(self):
        return "Sinch SDK client"



from dataclasses import dataclass
from sinch.core.models.base_model import SinchRequestBaseModel


@dataclass
class GetVoiceConferenceRequest(SinchRequestBaseModel):
    conference_id: str


@dataclass
class KickAllVoiceConferenceRequest(SinchRequestBaseModel):
    conference_id: str


@dataclass
class ManageParticipantVoiceConferenceRequest(SinchRequestBaseModel):
    conference_id: str
    call_id: str
    command: str
    moh: str


@dataclass
class KickParticipantVoiceConferenceRequest(SinchRequestBaseModel):
    conference_id: str
    call_id: str



from dataclasses import dataclass
from typing import List
from sinch.core.models.base_model import SinchBaseModel
from sinch.domains.conversation.models.conversation import Conversation


@dataclass
class SinchCreateConversationResponse(Conversation):
    pass


@dataclass
class SinchListConversationsResponse(SinchBaseModel):
    conversations: List[Conversation]
    next_page_token: str
    total_size: int


@dataclass
class SinchGetConversationResponse(Conversation):
    pass


@dataclass
class SinchDeleteConversationResponse(SinchBaseModel):
    pass


@dataclass
class SinchUpdateConversationResponse(Conversation):
    pass


@dataclass
class SinchStopConversationResponse(SinchBaseModel):
    pass


@dataclass
class SinchInjectMessageResponse(SinchBaseModel):
    pass



from dataclasses import dataclass
from typing import List
from sinch.core.models.base_model import SinchBaseModel
from sinch.domains.conversation.models import SinchConversationContact


@dataclass
class ListConversationContactsResponse(SinchBaseModel):
    contacts: List[SinchConversationContact]
    next_page_token: str


@dataclass
class CreateConversationContactResponse(SinchConversationContact):
    pass


@dataclass
class DeleteConversationContactResponse(SinchBaseModel):
    pass


@dataclass
class GetConversationContactResponse(SinchConversationContact):
    pass


@dataclass
class MergeConversationContactsResponse(SinchConversationContact):
    pass


@dataclass
class GetConversationChannelProfileResponse(SinchBaseModel):
    profile_name: str


@dataclass
class UpdateConversationContactResponse(SinchConversationContact):
    pass



from sinch.core.models.http_response import HTTPResponse
from sinch.domains.sms.endpoints.sms_endpoint import SMSEndpoint
from sinch.core.enums import HTTPAuthentication, HTTPMethods
from sinch.domains.sms.models.groups.requests import DeleteSMSGroupRequest
from sinch.domains.sms.models.groups.responses import SinchDeleteSMSGroupResponse


class DeleteSMSGroupEndpoint(SMSEndpoint):
    ENDPOINT_URL = "{origin}/xms/v1/{project_or_service_id}/groups/{group_id}"
    HTTP_METHOD = HTTPMethods.DELETE.value
    HTTP_AUTHENTICATION = HTTPAuthentication.OAUTH.value

    def __init__(self, request_data: DeleteSMSGroupRequest, sinch):
        super().__init__(request_data, sinch)

    def build_url(self, sinch) -> str:
        return self.ENDPOINT_URL.format(
            origin=sinch.configuration.sms_origin,
            project_or_service_id=self.project_or_service_id,
            group_id=self.request_data.group_id
        )

    def handle_response(self, response: HTTPResponse):
        super(DeleteSMSGroupEndpoint, self).handle_response(response)
        return SinchDeleteSMSGroupResponse()



from dataclasses import dataclass
from typing import List

from sinch.domains.conversation.models.webhook import ConversationWebhook
from sinch.core.models.base_model import SinchBaseModel


@dataclass
class CreateWebhookResponse(ConversationWebhook):
    pass


@dataclass
class UpdateWebhookResponse(ConversationWebhook):
    pass


@dataclass
class SinchListWebhooksResponse(SinchBaseModel):
    webhooks: List[ConversationWebhook]


@dataclass
class GetWebhookResponse(ConversationWebhook):
    pass


@dataclass
class SinchDeleteWebhookResponse(SinchBaseModel):
    pass



from sinch.core.pagination import TokenBasedPaginator, AsyncTokenBasedPaginator
from sinch.domains.numbers.endpoints.available.search_for_number import SearchForNumberEndpoint
from sinch.domains.numbers.endpoints.available.list_available_numbers import AvailableNumbersEndpoint
from sinch.domains.numbers.endpoints.available.activate_number import ActivateNumberEndpoint
from sinch.domains.numbers.endpoints.available.rent_any_number import RentAnyNumberEndpoint
from sinch.domains.numbers.endpoints.callbacks.get_configuration import GetNumbersCallbackConfigurationEndpoint
from sinch.domains.numbers.endpoints.callbacks.update_configuration import UpdateNumbersCallbackConfigurationEndpoint

from sinch.domains.numbers.endpoints.active.list_active_numbers_for_project import ListActiveNumbersEndpoint
from sinch.domains.numbers.endpoints.active.update_number_configuration import UpdateNumberConfigurationEndpoint
from sinch.domains.numbers.endpoints.active.get_number_configuration import GetNumberConfigurationEndpoint
from sinch.domains.numbers.endpoints.active.release_number_from_project import ReleaseNumberFromProjectEndpoint
from sinch.domains.numbers.endpoints.regions.list_available_regions import ListAvailableRegionsEndpoint

from sinch.domains.numbers.models.regions.requests import ListAvailableRegionsForProjectRequest
from sinch.domains.numbers.models.active.requests import (
    ListActiveNumbersRequest, GetNumberConfigurationRequest,
    UpdateNumberConfigurationRequest, ReleaseNumberFromProjectRequest
)
from sinch.domains.numbers.models.available.requests import (
    ListAvailableNumbersRequest, ActivateNumberRequest,
    CheckNumberAvailabilityRequest, RentAnyNumberRequest
)
from sinch.domains.numbers.models.regions.responses import ListAvailableRegionsResponse
from sinch.domains.numbers.models.available.responses import (
    ListAvailableNumbersResponse, ActivateNumberResponse,
    CheckNumberAvailabilityResponse
)
from sinch.domains.numbers.models.active.responses import (
    ListActiveNumbersResponse, UpdateNumberConfigurationResponse,
    GetNumberConfigurationResponse, ReleaseNumberFromProjectResponse
)
from sinch.domains.numbers.models.callbacks.responses import (
    GetNumbersCallbackConfigurationResponse,
    UpdateNumbersCallbackConfigurationResponse
)
from sinch.domains.numbers.models.callbacks.requests import (
    UpdateNumbersCallbackConfigurationRequest
)


class AvailableNumbers:
    def __init__(self, sinch):
        self._sinch = sinch

    def list(
        self,
        region_code: str,
        number_type: str,
        number_pattern: str = None,
        number_search_pattern: str = None,
        capabilities: list = None,
        page_size: int = None
    ) -> ListAvailableNumbersResponse:
        """
        Search for available virtual numbers using a variety of parameters to filter results.
        For additional documentation, see https://www.sinch.com and visit our developer portal.
        """
        return self._sinch.configuration.transport.request(
            AvailableNumbersEndpoint(
                project_id=self._sinch.configuration.project_id,
                request_data=ListAvailableNumbersRequest(
                    region_code=region_code,
                    number_type=number_type,
                    page_size=page_size,
                    capabilities=capabilities,
                    number_search_pattern=number_search_pattern,
                    number_pattern=number_pattern
                )
            )
        )

    def activate(
        self,
        phone_number: str,
        sms_configuration: dict = None,
        voice_configuration: dict = None
    ) -> ActivateNumberResponse:
        """
        Activate a virtual number to use with SMS products, Voice products, or both.
        For additional documentation, see https://www.sinch.com and visit our developer portal.
        """
        return self._sinch.configuration.transport.request(
            ActivateNumberEndpoint(
                project_id=self._sinch.configuration.project_id,
                request_data=ActivateNumberRequest(
                    phone_number=phone_number,
                    sms_configuration=sms_configuration,
                    voice_configuration=voice_configuration
                )
            )
        )

    def rent_any(
        self,
        region_code: str,
        type_: str,
        number_pattern: str = None,
        capabilities: list = None,
        sms_configuration: dict = None,
        voice_configuration: dict = None,
        callback_url: str = None
    ) -> RentAnyNumberRequest:
        return self._sinch.configuration.transport.request(
            RentAnyNumberEndpoint(
                project_id=self._sinch.configuration.project_id,
                request_data=RentAnyNumberRequest(
                    region_code=region_code,
                    type_=type_,
                    number_pattern=number_pattern,
                    capabilities=capabilities,
                    sms_configuration=sms_configuration,
                    voice_configuration=voice_configuration,
                    callback_url=callback_url
                )
            )
        )

    def check_availability(self, phone_number: str) -> CheckNumberAvailabilityResponse:
        """
        Enter a specific phone number to check availability.
        For additional documentation, see https://www.sinch.com and visit our developer portal.
        """
        return self._sinch.configuration.transport.request(
            SearchForNumberEndpoint(
                project_id=self._sinch.configuration.project_id,
                request_data=CheckNumberAvailabilityRequest(
                    phone_number=phone_number
                )
            )
        )


class ActiveNumbers:
    def __init__(self, sinch):
        self._sinch = sinch

    def list(
        self,
        region_code: str,
        number_type: str,
        number_pattern: str = None,
        number_search_pattern: str = None,
        capabilities: list = None,
        page_size: int = None,
        page_token: str = None
    ) -> ListActiveNumbersResponse:
        """
        Search for all active virtual numbers associated with a certain project.
        For additional documentation, see https://www.sinch.com and visit our developer portal.
        """
        return TokenBasedPaginator._initialize(
            sinch=self._sinch,
            endpoint=ListActiveNumbersEndpoint(
                project_id=self._sinch.configuration.project_id,
                request_data=ListActiveNumbersRequest(
                    region_code=region_code,
                    number_type=number_type,
                    page_size=page_size,
                    capabilities=capabilities,
                    number_pattern=number_pattern,
                    number_search_pattern=number_search_pattern,
                    page_token=page_token
                )
            )
        )

    def update(
        self,
        phone_number: str = None,
        display_name: str = None,
        sms_configuration: dict = None,
        voice_configuration: dict = None,
        app_id: str = None
    ) -> UpdateNumberConfigurationResponse:
        """
        Make updates to the configuration of your virtual number.
        Update the display name, change the currency type, or reconfigure for either SMS and/or Voice.
        For additional documentation, see https://www.sinch.com and visit our developer portal.
        """
        return self._sinch.configuration.transport.request(
            UpdateNumberConfigurationEndpoint(
                project_id=self._sinch.configuration.project_id,
                request_data=UpdateNumberConfigurationRequest(
                    phone_number=phone_number,
                    display_name=display_name,
                    sms_configuration=sms_configuration,
                    voice_configuration=voice_configuration,
                    app_id=app_id
                )
            )
        )

    def get(self, phone_number: str) -> GetNumberConfigurationResponse:
        """
        List of configuration settings for your virtual number.
        For additional documentation, see https://www.sinch.com and visit our developer portal.
        """
        return self._sinch.configuration.transport.request(
            GetNumberConfigurationEndpoint(
                project_id=self._sinch.configuration.project_id,
                request_data=GetNumberConfigurationRequest(
                    phone_number=phone_number
                )
            )
        )

    def release(self, phone_number: str) -> ReleaseNumberFromProjectResponse:
        """
        Release numbers you no longer need from your project.
        For additional documentation, see https://www.sinch.com and visit our developer portal.
        """
        return self._sinch.configuration.transport.request(
            ReleaseNumberFromProjectEndpoint(
                project_id=self._sinch.configuration.project_id,
                request_data=ReleaseNumberFromProjectRequest(
                    phone_number=phone_number
                )
            )
        )


class ActiveNumbersWithAsyncPagination(ActiveNumbers):
    async def list(
        self,
        region_code: str,
        number_type: str,
        number_pattern: str = None,
        number_search_pattern: str = None,
        capabilities: list = None,
        page_size: int = None,
        page_token: str = None
    ) -> ListActiveNumbersResponse:
        return await AsyncTokenBasedPaginator._initialize(
            sinch=self._sinch,
            endpoint=ListActiveNumbersEndpoint(
                project_id=self._sinch.configuration.project_id,
                request_data=ListActiveNumbersRequest(
                    region_code=region_code,
                    number_type=number_type,
                    page_size=page_size,
                    capabilities=capabilities,
                    number_pattern=number_pattern,
                    number_search_pattern=number_search_pattern,
                    page_token=page_token
                )
            )
        )


class AvailableRegions:
    def __init__(self, sinch):
        self._sinch = sinch

    def list(
        self,
        number_type: str = None,
        number_types: list = None
    ) -> ListAvailableRegionsResponse:
        """
        Lists all regions for numbers provided using the project ID.
        Some numbers can be configured for multiple regions.
        See which regions apply to your virtual number.
        For additional documentation, see https://www.sinch.com and visit our developer portal.
        """
        return self._sinch.configuration.transport.request(
            ListAvailableRegionsEndpoint(
                project_id=self._sinch.configuration.project_id,
                request_data=ListAvailableRegionsForProjectRequest(
                    number_type=number_type,
                    number_types=number_types
                )
            )
        )


class Callbacks:
    def __init__(self, sinch):
        self._sinch = sinch

    def get_configuration(self) -> GetNumbersCallbackConfigurationResponse:
        return self._sinch.configuration.transport.request(
            GetNumbersCallbackConfigurationEndpoint(
                project_id=self._sinch.configuration.project_id
            )
        )

    def update_configuration(self, hmac_secret) -> UpdateNumbersCallbackConfigurationResponse:
        return self._sinch.configuration.transport.request(
            UpdateNumbersCallbackConfigurationEndpoint(
                project_id=self._sinch.configuration.project_id,
                request_data=UpdateNumbersCallbackConfigurationRequest(
                    hmac_secret=hmac_secret
                )
            )
        )


class NumbersBase:
    """
    Documentation for Sinch virtual Numbers is found at https://developers.sinch.com/docs/numbers/.
    """
    def __init__(self, sinch):
        self._sinch = sinch


class Numbers(NumbersBase):
    """
    Synchronous version of the Numbers Domain
    """
    __doc__ += NumbersBase.__doc__

    def __init__(self, sinch):
        super(Numbers, self).__init__(sinch)
        self.available = AvailableNumbers(self._sinch)
        self.regions = AvailableRegions(self._sinch)
        self.active = ActiveNumbers(self._sinch)
        self.callbacks = Callbacks(self._sinch)


class NumbersAsync(NumbersBase):
    """
    Asynchronous version of the Numbers Domain
    """
    __doc__ += NumbersBase.__doc__

    def __init__(self, sinch):
        super(NumbersAsync, self).__init__(sinch)
        self.available = AvailableNumbers(self._sinch)
        self.regions = AvailableRegions(self._sinch)
        self.active = ActiveNumbersWithAsyncPagination(self._sinch)
        self.callbacks = Callbacks(self._sinch)



from sinch.core.models.http_response import HTTPResponse
from sinch.domains.conversation.endpoints.conversation_endpoint import ConversationEndpoint
from sinch.core.enums import HTTPAuthentication, HTTPMethods
from sinch.domains.conversation.models.webhook.responses import SinchListWebhooksResponse
from sinch.domains.conversation.models.webhook.requests import ListConversationWebhookRequest
from sinch.domains.conversation.models.webhook import ConversationWebhook


class ListWebhooksEndpoint(ConversationEndpoint):
    ENDPOINT_URL = "{origin}/v1/projects/{project_id}/apps/{app_id}/webhooks"
    HTTP_METHOD = HTTPMethods.GET.value
    HTTP_AUTHENTICATION = HTTPAuthentication.OAUTH.value

    def __init__(self, project_id: str, request_data: ListConversationWebhookRequest):
        super(ListWebhooksEndpoint, self).__init__(project_id, request_data)
        self.request_data = request_data
        self.project_id = project_id

    def build_url(self, sinch):
        return self.ENDPOINT_URL.format(
            origin=sinch.configuration.conversation_origin,
            project_id=self.project_id,
            app_id=self.request_data.app_id
        )

    def handle_response(self, response: HTTPResponse) -> SinchListWebhooksResponse:
        super(ListWebhooksEndpoint, self).handle_response(response)
        return SinchListWebhooksResponse(
            webhooks=[ConversationWebhook(
                id=webhook["id"],
                app_id=webhook["app_id"],
                target=webhook["target"],
                target_type=webhook["target_type"],
                secret=webhook["secret"],
                triggers=webhook["triggers"],
                client_credentials=webhook["client_credentials"]
            ) for webhook in response.body["webhooks"]]
        )



from dataclasses import dataclass

from sinch.core.models.base_model import SinchRequestBaseModel


@dataclass
class WebhookRequest(SinchRequestBaseModel):
    app_id: str
    target: str
    triggers: list
    client_credentials: dict
    secret: str
    target_type: str


@dataclass
class CreateConversationWebhookRequest(WebhookRequest):
    pass


@dataclass
class ListConversationWebhookRequest(SinchRequestBaseModel):
    app_id: str


@dataclass
class GetConversationWebhookRequest(SinchRequestBaseModel):
    webhook_id: str


@dataclass
class DeleteConversationWebhookRequest(SinchRequestBaseModel):
    webhook_id: str


@dataclass
class UpdateConversationWebhookRequest(WebhookRequest):
    update_mask: str
    webhook_id: str



from sinch.core.models.http_response import HTTPResponse
from sinch.core.endpoint import HTTPEndpoint
from sinch.core.enums import HTTPAuthentication, HTTPMethods
from sinch.domains.authentication.exceptions import AuthenticationException
from sinch.domains.authentication.models.authentication import OAuthToken


class OAuthEndpoint(HTTPEndpoint):
    ENDPOINT_URL = "{origin}/oauth2/token"
    HTTP_METHOD = HTTPMethods.POST.value
    HTTP_AUTHENTICATION = HTTPAuthentication.BASIC.value

    def __init__(self):
        pass

    def build_url(self, sinch) -> str:
        return self.ENDPOINT_URL.format(
            origin=sinch.configuration.auth_origin
        )

    def request_body(self) -> dict:
        return {
            "grant_type": "client_credentials"
        }

    def handle_response(self, response: HTTPResponse):
        if response.status_code >= 400:
            raise AuthenticationException(
                message=response.body.get("error_description"),
                response=response,
                is_from_server=True
            )
        else:
            return OAuthToken(
                access_token=response.body["access_token"],
                expires_in=response.body["expires_in"],
                scope=response.body["scope"],
                token_type=response.body["token_type"]
            )



from sinch.core.models.http_response import HTTPResponse
from sinch.domains.sms.endpoints.sms_endpoint import SMSEndpoint
from sinch.core.enums import HTTPAuthentication, HTTPMethods
from sinch.domains.sms.models.inbounds.requests import GetSMSInboundMessageRequest
from sinch.domains.sms.models.inbounds.responses import GetInboundMessagesResponse


class GetInboundMessagesEndpoint(SMSEndpoint):
    ENDPOINT_URL = "{origin}/xms/v1/{project_or_service_id}/inbounds/{inbound_id}"
    HTTP_METHOD = HTTPMethods.GET.value
    HTTP_AUTHENTICATION = HTTPAuthentication.OAUTH.value

    def __init__(self, request_data: GetSMSInboundMessageRequest, sinch):
        super().__init__(request_data, sinch)

    def build_url(self, sinch) -> str:
        return self.ENDPOINT_URL.format(
            origin=sinch.configuration.sms_origin,
            project_or_service_id=self.project_or_service_id,
            inbound_id=self.request_data.inbound_id
        )

    def handle_response(self, response: HTTPResponse):
        super(GetInboundMessagesEndpoint, self).handle_response(response)
        return GetInboundMessagesResponse(
            type=response.body.get("type"),
            id=response.body.get("id"),
            origin_number=response.body.get("from"),
            destination_number=response.body.get("to"),
            body=response.body.get("body"),
            operator_id=response.body.get("operator_id"),
            send_at=response.body.get("send_at"),
            received_at=response.body.get("received_at")
        )



from dataclasses import dataclass
from typing import Optional

from sinch.core.models.base_model import SinchBaseModel
from sinch.domains.conversation.enums import (
    ConversationChannel,
    ConversationRetentionPolicyType
)


@dataclass
class SinchConversationRecipient(SinchBaseModel):
    contact_id: str


@dataclass
class SinchConversationTextMessage(SinchBaseModel):
    pass


@dataclass
class SinchConversationMessage(SinchBaseModel):
    id: str
    direction: str
    channel_identity: str
    app_message: dict
    conversation_id: str
    contact_id: str
    metadata: str
    accept_time: str
    sender_id: str
    processing_mode: str


@dataclass
class SinchConversationChannelIdentities(SinchBaseModel):
    channel: ConversationChannel
    identity: str
    app_id: str


@dataclass
class SinchConversationContact(SinchBaseModel):
    id: str
    channel_identities: SinchConversationChannelIdentities
    channel_priority: list
    display_name: str
    email: str
    external_id: str
    metadata: str
    language: str


@dataclass
class SinchConversationRetentionPolicy(SinchBaseModel):
    retention_type: ConversationRetentionPolicyType
    ttl_days: int


@dataclass
class SinchConversationTelegramCredentials(SinchBaseModel):  # TODO: add more communication channels
    token: str


@dataclass
class SinchConversationChannelCredentials(SinchBaseModel):
    channel: ConversationChannel
    callback_secret: Optional[str] = None
    telegram_credentials: Optional[SinchConversationTelegramCredentials] = None


@dataclass
class SinchConversationApp(SinchBaseModel):
    id: str
    channel_credentials: dict
    processing_mode: str
    conversation_metadata_report_view: str
    display_name: str
    rate_limits: dict
    retention_policy: dict
    dispatch_retention_policy: dict
    smart_conversation: dict



from sinch.core.models.http_response import HTTPResponse
from sinch.domains.verification.endpoints.verification_endpoint import VerificationEndpoint
from sinch.core.enums import HTTPAuthentication, HTTPMethods
from sinch.domains.verification.enums import VerificationMethod
from sinch.domains.verification.models.requests import StartVerificationRequest
from sinch.domains.verification.models.responses import (
    FlashCallResponse,
    SMSResponse,
    DataResponse,
    StartVerificationResponse,
    StartSMSVerificationResponse,
    StartDataVerificationResponse,
    StartPhoneCallVerificationResponse,
    StartFlashCallVerificationResponse
)


class StartVerificationEndpoint(VerificationEndpoint):
    ENDPOINT_URL = "{origin}/verification/v1/verifications"
    HTTP_METHOD = HTTPMethods.POST.value
    HTTP_AUTHENTICATION = HTTPAuthentication.SIGNED.value

    def __init__(self, request_data: StartVerificationRequest):
        self.request_data = request_data

    def build_url(self, sinch):
        return self.ENDPOINT_URL.format(
            origin=sinch.configuration.verification_origin,
        )

    def request_body(self):
        return self.request_data.as_json()

    def handle_response(self, response: HTTPResponse) -> StartVerificationResponse:
        super().handle_response(response)
        if self.request_data.method == VerificationMethod.SMS.value:
            sms_response = response.body.get("sms")
            return StartSMSVerificationResponse(
                id=response.body.get("id"),
                method=response.body.get("method"),
                _links=response.body.get("_links"),
                sms=SMSResponse(
                    interception_timeout=response.body["sms"].get("interceptionTimeout"),
                    template=response.body["sms"].get("template")
                ) if sms_response else None
            )
        elif self.request_data.method == VerificationMethod.FLASH_CALL.value:
            flash_call_response = response.body.get("flashCall")
            return StartFlashCallVerificationResponse(
                id=response.body.get("id"),
                method=response.body.get("method"),
                _links=response.body.get("_links"),
                flash_call=FlashCallResponse(
                    cli_filter=response.body["flashCall"].get("cliFilter"),
                    interception_timeout=response.body["flashCall"].get("interceptionTimeout"),
                    report_timeout=response.body["flashCall"].get("reportTimeout"),
                    deny_call_after=response.body["flashCall"].get("denyCallAfter")
                ) if flash_call_response else None
            )
        elif self.request_data.method == VerificationMethod.CALLOUT.value:
            return StartPhoneCallVerificationResponse(
                id=response.body.get("id"),
                method=response.body.get("method"),
                _links=response.body.get("_links")
            )
        elif self.request_data.method == VerificationMethod.SEAMLESS.value:
            seamless_response = response.body.get("seamless")
            return StartDataVerificationResponse(
                id=response.body.get("id"),
                method=response.body.get("method"),
                _links=response.body.get("_links"),
                seamless=DataResponse(
                    target_uri=response.body["seamless"].get("targetUri")
                ) if seamless_response else None
            )



from enum import Enum


class CalloutMethod(Enum):
    TEXT_TO_SPEECH = "ttsCallout"
    CUSTOM = "customCallout"
    CONFERENCE = "conferenceCallout"


class Region(Enum):
    EUROPE = "euc1"
    NORTH_AMERICA = "use1"
    SOUTH_AMERICA = "sae1"
    SOUTH_EAST_ASIA_1 = "apse1"
    SOUTH_EAST_ASIA_2 = "apse2"


class ConferenceCommand(Enum):
    MUTE = "mute"
    UNMUTE = "unmute"
    ONHOLD = "onhold"
    RESUME = "resume"


class MusicOnHold(Enum):
    RING = "ring"
    MUSIC_1 = "music1"
    MUSIC_2 = "music2"
    MUSIC_3 = "music3"


class ConferenceDTMFOptionsMode(Enum):
    IGNORE = "ignore"
    FORWARD = "forward"
    DETECT = "detect"


class Indications(Enum):
    AUSTRIA = "at"
    AUSTRALIA = "au"
    BULGARIA = "bg"
    BRAZIL = "br"
    BELGIUM = "be"
    SWITZERLAND = "ch"
    CHILE = "cl"
    CHINA = "cn"
    CZECH_REPUBLIC = "cz"
    GERMANY = "de"
    DENMARK = "dk"
    ESTONIA = "ee"
    SPAIN = "es"
    FINLAND = "fi"
    FRANCE = "fr"
    GREECE = "gr"
    HUNGARY = "hu"
    ISRAEL = "il"
    INDIA = "in"
    ITALY = "it"
    LITHUANIA = "lt"
    JAPAN = "jp"
    MEXICO = "mx"
    MALAYSIA = "my"
    NETHERLANDS = "nl"
    NORWAY = "no"
    NEW_ZEALAND = "nz"
    PHILIPPINES = "ph"
    POLAND = "pl"
    PORTUGAL = "pt"
    RUSSIA = "ru"
    SWEDEN = "se"
    SINGAPORE = "sg"
    THAILAND = "th"
    UNITED_KINGDOM = "uk"
    UNITED_STATES = "us"
    TAIWAN = "tw"
    VENEZUELA = "ve"
    SOUTH_AFRICA = "za"


class Capability(Enum):
    VOCE = "voice"
    SMS = "sms"



from sinch.core.models.http_response import HTTPResponse
from sinch.domains.sms.endpoints.sms_endpoint import SMSEndpoint
from sinch.core.enums import HTTPAuthentication, HTTPMethods
from sinch.domains.sms.models.delivery_reports.requests import GetSMSDeliveryReportForNumberRequest
from sinch.domains.sms.models.delivery_reports.responses import GetSMSDeliveryReportForNumberResponse


class GetDeliveryReportForNumberEndpoint(SMSEndpoint):
    ENDPOINT_URL = "{origin}/xms/v1/{project_or_service_id}/batches/{batch_id}/delivery_report/{recipient_msisdn}"
    HTTP_METHOD = HTTPMethods.GET.value
    HTTP_AUTHENTICATION = HTTPAuthentication.OAUTH.value

    def __init__(self, request_data: GetSMSDeliveryReportForNumberRequest, sinch):
        super().__init__(request_data, sinch)

    def build_url(self, sinch) -> str:
        return self.ENDPOINT_URL.format(
            origin=sinch.configuration.sms_origin,
            project_or_service_id=self.project_or_service_id,
            batch_id=self.request_data.batch_id,
            recipient_msisdn=self.request_data.recipient_number
        )

    def handle_response(self, response: HTTPResponse):
        super(GetDeliveryReportForNumberEndpoint, self).handle_response(response)
        return GetSMSDeliveryReportForNumberResponse(
            at=response.body.get("at"),
            batch_id=response.body.get("batch_id"),
            code=response.body.get("code"),
            recipient=response.body.get("recipient"),
            status=response.body.get("status"),
            applied_originator=response.body.get("applied_originator"),
            client_reference=response.body.get("client_reference"),
            number_of_message_parts=response.body.get("number_of_message_parts"),
            operator=response.body.get("operator"),
            operator_status_at=response.body.get("operator_status_at"),
            type=response.body.get("type")
        )



from abc import ABC, abstractmethod
from sinch.core.models.http_response import HTTPResponse


class HTTPEndpoint(ABC):
    ENDPOINT_URL = None
    HTTP_METHOD = None
    HTTP_AUTHENTICATION = None

    def __init__(self, project_id, request_data):
        pass

    def get_url_without_origin(self, sinch):
        return '/' + '/'.join(self.build_url(sinch).split('/')[1:])

    def build_url(self, sinch):
        return

    def build_query_params(self):
        pass

    def request_body(self):
        return

    @abstractmethod
    def handle_response(self, response: HTTPResponse):
        pass



from dataclasses import dataclass
from typing import Optional, List
from sinch.core.models.base_model import SinchRequestBaseModel


@dataclass
class Instruction(SinchRequestBaseModel):
    pass


@dataclass
class TranscriptionOptions(SinchRequestBaseModel):
    enabled: str = None
    locale: str = None


@dataclass
class RecordingOptions(SinchRequestBaseModel):
    destination_url: str = None
    credentials: str = None
    format: str = None
    notification_events: str = None
    transcription_options: TranscriptionOptions = None

    def as_dict(self):
        payload = super().as_dict()
        if payload.get("destination_url"):
            payload["destinationUrl"] = payload.pop("destination_url")

        if payload.get("notification_events"):
            payload["notificationEvents"] = payload.pop("notification_events")

        if payload.get("transcription_options"):
            payload["transcriptionOptions"] = payload.pop("transcription_options")

        return payload


@dataclass
class PlayFileInstruction(Instruction):
    ids: List[List[str]]
    locale: str
    name: str = "playFiles"


@dataclass
class SayInstruction(Instruction):
    name: str = "say"
    text: Optional[str] = None
    locale: Optional[str] = None


@dataclass
class SendDtmfInstruction(Instruction):
    name: str = "sendDtmf"
    value: Optional[str] = None


@dataclass
class SetCookieInstruction(Instruction):
    name: str = "setCookie"
    key: Optional[str] = None
    value: Optional[str] = None


@dataclass
class AnswerInstruction(Instruction):
    name: str = "answer"


@dataclass
class StartRecordingInstruction(Instruction):
    name: str = "startRecording"
    options: Optional[RecordingOptions] = None


@dataclass
class StopRecordingInstruction(Instruction):
    name: str = "stopRecording"



from sinch.core.models.http_response import HTTPResponse
from sinch.domains.numbers.endpoints.numbers_endpoint import NumbersEndpoint
from sinch.core.enums import HTTPAuthentication, HTTPMethods
from sinch.domains.numbers.models.regions import Region

from sinch.domains.numbers.models.regions.responses import ListAvailableRegionsResponse
from sinch.domains.numbers.models.regions.requests import ListAvailableRegionsForProjectRequest


class ListAvailableRegionsEndpoint(NumbersEndpoint):
    ENDPOINT_URL = "{origin}/v1/projects/{project_id}/availableRegions"
    HTTP_METHOD = HTTPMethods.GET.value
    HTTP_AUTHENTICATION = HTTPAuthentication.OAUTH.value

    def __init__(self, project_id: str, request_data: ListAvailableRegionsForProjectRequest):
        super(ListAvailableRegionsEndpoint, self).__init__(project_id, request_data)
        self.project_id = project_id
        self.request_data = request_data

    def build_url(self, sinch):
        return self.ENDPOINT_URL.format(
            origin=sinch.configuration.numbers_origin,
            project_id=self.project_id
        )

    def build_query_params(self) -> dict:
        query_params = {}
        if self.request_data.number_type:
            query_params["type"] = self.request_data.number_type

        if self.request_data.number_types:
            query_params["types"] = self.request_data.number_types

        return query_params

    def handle_response(self, response: HTTPResponse) -> ListAvailableRegionsResponse:
        super(ListAvailableRegionsEndpoint, self).handle_response(response)
        return ListAvailableRegionsResponse(
            [
                Region(
                    region_code=region["regionCode"],
                    region_name=region["regionName"],
                    types=region["types"]
                ) for region in response.body["availableRegions"]
            ]
        )



import json
from sinch.core.models.http_response import HTTPResponse
from sinch.domains.numbers.endpoints.numbers_endpoint import NumbersEndpoint
from sinch.core.enums import HTTPAuthentication, HTTPMethods
from sinch.domains.numbers.models.available.requests import RentAnyNumberRequest
from sinch.domains.numbers.models.available.responses import RentAnyNumberResponse


class RentAnyNumberEndpoint(NumbersEndpoint):
    ENDPOINT_URL = "{origin}/v1/projects/{project_id}/availableNumbers:rentAny"
    HTTP_METHOD = HTTPMethods.POST.value
    HTTP_AUTHENTICATION = HTTPAuthentication.OAUTH.value

    def __init__(self, project_id: str, request_data: RentAnyNumberRequest):
        super(RentAnyNumberEndpoint, self).__init__(project_id, request_data)
        self.project_id = project_id
        self.request_data = request_data

    def build_url(self, sinch) -> str:
        return self.ENDPOINT_URL.format(
            origin=sinch.configuration.numbers_origin,
            project_id=self.project_id
        )

    def request_body(self):
        request_data = self.request_data.as_dict()
        request_body = {}

        if request_data.get("region_code"):
            request_body["regionCode"] = request_data["region_code"]

        if request_data.get("type_"):
            request_body["type"] = request_data["type_"]

        if request_data.get("number_pattern"):
            request_body["numberPattern"] = request_data["number_pattern"]

        if request_data.get("capabilities"):
            request_body["capabilities"] = request_data["capabilities"]

        if request_data.get("sms_configuration"):
            request_body["smsConfiguration"] = request_data["sms_configuration"]

        if request_data.get("voice_configuration"):
            request_body["voiceConfiguration"] = request_data["voice_configuration"]

        if request_data.get("callback_url"):
            request_body["callbackUrl"] = request_data["callback_url"]

        return json.dumps(request_body)

    def handle_response(self, response: HTTPResponse) -> RentAnyNumberResponse:
        super(RentAnyNumberEndpoint, self).handle_response(response)
        return RentAnyNumberResponse(
            phone_number=response.body["phoneNumber"],
            region_code=response.body["regionCode"],
            type=response.body["type"],
            capability=response.body["capability"],
            project_id=response.body["projectId"],
            callback_url=response.body["callbackUrl"],
            expire_at=response.body["expireAt"],
            money=response.body["money"],
            next_charge_date=response.body["nextChargeDate"],
            sms_configuration=response.body["smsConfiguration"],
            voice_configuration=response.body["voiceConfiguration"],
            payment_interval_months=response.body["paymentIntervalMonths"]
        )



from dataclasses import dataclass
from sinch.core.models.base_model import SinchRequestBaseModel


@dataclass
class Batch(SinchRequestBaseModel):
    id: str
    to: list
    from_: str
    body: str
    delivery_report: str
    cancelled: str
    type: str
    campaign_id: str
    created_at: str
    modified_at: str
    send_at: str
    expire_at: str
    callback_url: str = None
    client_reference: str = None
    feedback_enabled: bool = None
    flash_message: bool = None



from sinch.core.models.http_response import HTTPResponse
from sinch.domains.voice.endpoints.voice_endpoint import VoiceEndpoint
from sinch.core.enums import HTTPAuthentication, HTTPMethods
from sinch.domains.voice.models.applications.requests import QueryNumberVoiceApplicationRequest
from sinch.domains.voice.models.applications.responses import QueryNumberVoiceApplicationResponse


class QueryVoiceNumberEndpoint(VoiceEndpoint):
    ENDPOINT_URL = "{origin}/v1/calling/query/number/{number}"
    HTTP_METHOD = HTTPMethods.GET.value
    HTTP_AUTHENTICATION = HTTPAuthentication.SIGNED.value

    def __init__(self, request_data: QueryNumberVoiceApplicationRequest):
        self.request_data = request_data

    def build_url(self, sinch) -> str:
        return self.ENDPOINT_URL.format(
            origin=sinch.configuration.voice_applications_origin,
            number=self.request_data.number
        )

    def handle_response(self, response: HTTPResponse) -> QueryNumberVoiceApplicationResponse:
        super().handle_response(response)
        return QueryNumberVoiceApplicationResponse(
            country_id=response.body["number"]["countryId"],
            number_type=response.body["number"]["numberType"],
            normalized_number=response.body["number"]["normalizedNumber"],
            restricted=response.body["number"]["restricted"],
            rate=response.body["number"]["rate"]
        )



from dataclasses import dataclass

from sinch.core.models.base_model import SinchBaseModel
from sinch.domains.numbers.enums import NumberType, NumberCapability


@dataclass
class ActiveNumber(SinchBaseModel):
    phone_number: str
    project_id: str
    display_name: str
    region_code: str
    type: NumberType
    capability: NumberCapability
    money: dict
    payment_interval_months: int
    next_charge_date: str
    expire_at: str
    sms_configuration: dict
    voice_configuration: dict



from sinch.domains.verification.endpoints.start_verification import StartVerificationEndpoint
from sinch.domains.verification.endpoints.report_verification_using_identity import (
    ReportVerificationByIdentityEndpoint
)
from sinch.domains.verification.endpoints.report_verification_using_id import (
    ReportVerificationByIdEndpoint
)
from sinch.domains.verification.endpoints.get_verification_by_identity import (
    GetVerificationStatusByIdentityEndpoint
)
from sinch.domains.verification.endpoints.get_verification_by_reference import (
    GetVerificationStatusByReferenceEndpoint
)
from sinch.domains.verification.endpoints.get_verification_by_id import (
    GetVerificationStatusByIdEndpoint
)
from sinch.domains.verification.models.responses import (
    StartVerificationResponse,
    ReportVerificationByIdentityResponse,
    ReportVerificationByIdResponse,
    GetVerificationStatusByIdentityResponse,
    GetVerificationStatusByReferenceResponse,
    GetVerificationStatusByIdResponse
)
from sinch.domains.verification.models.requests import (
    StartSMSVerificationRequest,
    StartFlashCallVerificationRequest,
    StartPhoneCallVerificationRequest,
    StartCalloutVerificationRequest,
    StartDataVerificationRequest,
    ReportVerificationByIdentityRequestLegacy,
    ReportVerificationByIdRequestLegacy,
    ReportVerificationByIdentityAndSMSRequest,
    ReportVerificationByIdentityAndFlashCallRequest,
    ReportVerificationByIdentityAndPhoneCallRequest,
    ReportVerificationByIdAndSMSRequest,
    ReportVerificationByIdAndFlashCallRequest,
    ReportVerificationByIdAndPhoneCallRequest,
    GetVerificationStatusByIdentityRequest,
    GetVerificationStatusByReferenceRequest,
    GetVerificationStatusByIdRequest
)
from sinch.domains.verification.models import VerificationIdentity


class Verifications:
    def __init__(self, sinch):
        self._sinch = sinch

    def start_sms(
        self,
        identity: VerificationIdentity,
        reference: str = None,
        custom: str = None,
        expiry: str = None,
        code_type: str = None,
        template: str = None
    ) -> StartVerificationResponse:
        return self._sinch.configuration.transport.request(
            StartVerificationEndpoint(
                request_data=StartSMSVerificationRequest(
                    identity=identity,
                    reference=reference,
                    custom=custom,
                    expiry=expiry,
                    code_type=code_type,
                    template=template
                )
            )
        )

    def start_flash_call(
        self,
        identity: VerificationIdentity,
        reference: str = None,
        dial_timeout: int = None,
        custom: str = None
    ) -> StartVerificationResponse:
        return self._sinch.configuration.transport.request(
            StartVerificationEndpoint(
                request_data=StartFlashCallVerificationRequest(
                    identity=identity,
                    reference=reference,
                    dial_timeout=dial_timeout,
                    custom=custom
                )
            )
        )

    def start_phone_call(
        self,
        identity: VerificationIdentity,
        reference: str = None,
        custom: str = None
    ) -> StartVerificationResponse:
        return self._sinch.configuration.transport.request(
            StartVerificationEndpoint(
                request_data=StartPhoneCallVerificationRequest(
                    identity=identity,
                    reference=reference,
                    custom=custom
                )
            )
        )

    def start_callout(
        self,
        identity: VerificationIdentity,
        reference: str = None,
        custom: str = None,
        speech_locale: str = None
    ) -> StartVerificationResponse:
        """
        This method is not supported anymore.
        It should be used only for backward compatibility reasons.
        Use start_phone_call method instead.
        """
        return self._sinch.configuration.transport.request(
            StartVerificationEndpoint(
                request_data=StartCalloutVerificationRequest(
                    identity=identity,
                    reference=reference,
                    custom=custom,
                    speech_locale=speech_locale
                )
            )
        )

    def start_seamless(
        self,
        identity: VerificationIdentity,
        reference: str = None,
        custom: str = None
    ) -> StartVerificationResponse:
        """
        This method is not supported anymore.
        It should be used only for backward compatibility reasons.
        Use start_data method instead.
        """
        return self._sinch.configuration.transport.request(
            StartVerificationEndpoint(
                request_data=StartDataVerificationRequest(
                    identity=identity,
                    reference=reference,
                    custom=custom
                )
            )
        )

    def start_data(
        self,
        identity: VerificationIdentity,
        reference: str = None,
        custom: str = None
    ) -> StartVerificationResponse:
        return self._sinch.configuration.transport.request(
            StartVerificationEndpoint(
                request_data=StartDataVerificationRequest(
                    identity=identity,
                    reference=reference,
                    custom=custom
                )
            )
        )

    def report_sms_by_id(
        self,
        id: str,
        code: str,
        cli: str = None
    ) -> ReportVerificationByIdResponse:
        return self._sinch.configuration.transport.request(
            ReportVerificationByIdEndpoint(
                request_data=ReportVerificationByIdAndSMSRequest(
                    id,
                    code,
                    cli
                )
            )
        )

    def report_flash_call_by_id(
        self,
        id: str,
        cli: str
    ) -> ReportVerificationByIdResponse:
        return self._sinch.configuration.transport.request(
            ReportVerificationByIdEndpoint(
                request_data=ReportVerificationByIdAndFlashCallRequest(
                    id,
                    cli
                )
            )
        )

    def report_phone_call_by_id(
        self,
        id: str,
        code: str = None
    ) -> ReportVerificationByIdResponse:
        return self._sinch.configuration.transport.request(
            ReportVerificationByIdEndpoint(
                request_data=ReportVerificationByIdAndPhoneCallRequest(
                    id,
                    code
                )
            )
        )

    def report_sms_by_identity(
        self,
        endpoint: str,
        code: str,
        cli: str = None
    ) -> ReportVerificationByIdentityResponse:
        return self._sinch.configuration.transport.request(
            ReportVerificationByIdentityEndpoint(
                request_data=ReportVerificationByIdentityAndSMSRequest(
                    endpoint,
                    code,
                    cli
                )
            )
        )

    def report_flash_call_by_identity(
        self,
        endpoint: str,
        cli: str = None
    ) -> ReportVerificationByIdentityResponse:
        return self._sinch.configuration.transport.request(
            ReportVerificationByIdentityEndpoint(
                request_data=ReportVerificationByIdentityAndFlashCallRequest(
                    endpoint,
                    cli
                )
            )
        )

    def report_phone_call_by_identity(
        self,
        endpoint: str,
        code: str
    ) -> ReportVerificationByIdentityResponse:
        return self._sinch.configuration.transport.request(
            ReportVerificationByIdentityEndpoint(
                request_data=ReportVerificationByIdentityAndPhoneCallRequest(
                    endpoint,
                    code
                )
            )
        )

    def report_by_id(
        self,
        id: str,
        verification_report_request: dict
    ) -> ReportVerificationByIdResponse:
        """
        This method is not supported anymore.
        It should be used only for backward compatibility reasons.
        """
        return self._sinch.configuration.transport.request(
            ReportVerificationByIdEndpoint(
                request_data=ReportVerificationByIdRequestLegacy(
                    id,
                    verification_report_request
                )
            )
        )

    def report_by_identity(
        self,
        endpoint,
        verification_report_request
    ) -> ReportVerificationByIdentityResponse:
        """
        This method is not supported anymore.
        It should be used only for backward compatibility reasons.
        """
        return self._sinch.configuration.transport.request(
            ReportVerificationByIdentityEndpoint(
                request_data=ReportVerificationByIdentityRequestLegacy(
                    endpoint,
                    verification_report_request
                )
            )
        )


class VerificationStatus:
    def __init__(self, sinch):
        self._sinch = sinch

    def get_by_id(self, id: str) -> GetVerificationStatusByIdResponse:
        return self._sinch.configuration.transport.request(
            GetVerificationStatusByIdEndpoint(
                request_data=GetVerificationStatusByIdRequest(
                    id=id
                )
            )
        )

    def get_by_reference(self, reference: str) -> GetVerificationStatusByReferenceResponse:
        return self._sinch.configuration.transport.request(
            GetVerificationStatusByReferenceEndpoint(
                request_data=GetVerificationStatusByReferenceRequest(
                    reference=reference
                )
            )
        )

    def get_by_identity(
        self,
        endpoint: str,
        method: str
    ) -> GetVerificationStatusByIdentityResponse:
        return self._sinch.configuration.transport.request(
            GetVerificationStatusByIdentityEndpoint(
                request_data=GetVerificationStatusByIdentityRequest(
                    endpoint=endpoint,
                    method=method
                )
            )
        )


class VerificationBase:
    """
    Documentation for the Verification API: https://developers.sinch.com/docs/verification/
    """
    def __init__(self, sinch):
        self._sinch = sinch


class Verification(VerificationBase):
    """
    Synchronous version of the Verification Domain
    """
    __doc__ += VerificationBase.__doc__

    def __init__(self, sinch):
        super(Verification, self).__init__(sinch)
        self.verifications = Verifications(self._sinch)
        self.verification_status = VerificationStatus(self._sinch)


class VerificationAsync(VerificationBase):
    """
    Asynchronous version of the Verification Domain
    """
    __doc__ += VerificationBase.__doc__

    def __init__(self, sinch):
        super(VerificationAsync, self).__init__(sinch)
        self.verifications = Verifications(self._sinch)
        self.verification_status = VerificationStatus(self._sinch)



from dataclasses import dataclass
from sinch.core.models.base_model import SinchBaseModel
from sinch.domains.verification.enums import VerificationMethod, VerificationStatus


@dataclass
class FlashCallResponse:
    cli_filter: str
    interception_timeout: int
    report_timeout: int
    deny_call_after: int


@dataclass
class SMSResponse:
    template: str
    interception_timeout: str


@dataclass
class DataResponse:
    target_uri: str


@dataclass
class StartVerificationResponse(SinchBaseModel):
    id: str
    method: VerificationMethod
    _links: list


@dataclass
class StartFlashCallInitiateVerificationResponse(StartVerificationResponse):
    flash_call: FlashCallResponse


@dataclass
class StartDataInitiateVerificationResponse(StartVerificationResponse):
    seamless: DataResponse


@dataclass
class StartCalloutInitiateVerificationResponse(StartVerificationResponse):
    pass


@dataclass
class StartSMSVerificationResponse(StartVerificationResponse):
    sms: SMSResponse


@dataclass
class StartFlashCallVerificationResponse(StartVerificationResponse):
    flash_call: FlashCallResponse


@dataclass
class StartDataVerificationResponse(StartVerificationResponse):
    seamless: DataResponse


@dataclass
class StartPhoneCallVerificationResponse(StartVerificationResponse):
    pass


@dataclass
class VerificationResponse(SinchBaseModel):
    id: str
    method: VerificationMethod
    status: VerificationStatus
    price: dict
    identity: dict
    country_id: str
    verification_timestamp: str
    reference: str
    reason: str
    call_complete: bool


@dataclass
class GetVerificationStatusByIdResponse(VerificationResponse):
    pass


@dataclass
class ReportVerificationResponse(VerificationResponse):
    pass


@dataclass
class ReportVerificationByIdentityResponse(ReportVerificationResponse):
    pass


@dataclass
class ReportVerificationByIdResponse(ReportVerificationResponse):
    pass


@dataclass
class GetVerificationStatusByReferenceResponse(VerificationResponse):
    pass


@dataclass
class GetVerificationStatusByIdentityResponse(VerificationResponse):
    pass



from sinch.core.models.http_response import HTTPResponse
from sinch.domains.sms.endpoints.sms_endpoint import SMSEndpoint
from sinch.core.enums import HTTPAuthentication, HTTPMethods
from sinch.domains.sms.models.groups.requests import GetSMSGroupPhoneNumbersRequest
from sinch.domains.sms.models.groups.responses import SinchGetSMSGroupPhoneNumbersResponse


class GetSMSGroupPhoneNumbersEndpoint(SMSEndpoint):
    ENDPOINT_URL = "{origin}/xms/v1/{project_or_service_id}/groups/{group_id}/members"
    HTTP_METHOD = HTTPMethods.GET.value
    HTTP_AUTHENTICATION = HTTPAuthentication.OAUTH.value

    def __init__(self, request_data: GetSMSGroupPhoneNumbersRequest, sinch):
        super().__init__(request_data, sinch)

    def build_url(self, sinch) -> str:
        return self.ENDPOINT_URL.format(
            origin=sinch.configuration.sms_origin,
            project_or_service_id=self.project_or_service_id,
            group_id=self.request_data.group_id
        )

    def handle_response(self, response: HTTPResponse):
        super(GetSMSGroupPhoneNumbersEndpoint, self).handle_response(response)
        return SinchGetSMSGroupPhoneNumbersResponse(
            phone_numbers=response.body
        )



from dataclasses import dataclass
from sinch.core.models.base_model import SinchRequestBaseModel


@dataclass
class TranscodeConversationMessageRequest(SinchRequestBaseModel):
    app_id: str
    app_message: dict
    channels: list
    from_: str
    to: str



import httpx
from sinch.core.ports.http_transport import AsyncHTTPTransport, HttpRequest
from sinch.core.endpoint import HTTPEndpoint
from sinch.core.models.http_response import HTTPResponse


class HTTPXTransport(AsyncHTTPTransport):
    def __init__(self, sinch):
        super().__init__(sinch)
        self.http_session = None

    async def request(self, endpoint: HTTPEndpoint) -> HTTPResponse:
        request_data: HttpRequest = self.prepare_request(endpoint)
        request_data: HttpRequest = await self.authenticate(endpoint, request_data)

        if not self.http_session:
            self.http_session = httpx.AsyncClient()

        self.sinch.configuration.logger.debug(
            f"Async HTTP {request_data.http_method} call with headers:"
            f" {request_data.headers} and body: {request_data.request_body} to URL: {request_data.url}"
        )

        if isinstance(request_data.request_body, str):
            response = await self.http_session.request(
                method=request_data.http_method,
                headers=request_data.headers,
                url=request_data.url,
                content=request_data.request_body,
                auth=request_data.auth,
                params=request_data.query_params,
                timeout=self.sinch.configuration.connection_timeout
            )
        else:
            response = await self.http_session.request(
                method=request_data.http_method,
                headers=request_data.headers,
                url=request_data.url,
                data=request_data.request_body,
                auth=request_data.auth,
                params=request_data.query_params,
                timeout=self.sinch.configuration.connection_timeout,
            )

        response_body = self.deserialize_json_response(response)

        self.sinch.configuration.logger.debug(
            f"Async HTTP {response.status_code} response with headers: {response.headers}"
            f"and body: {response_body} from URL: {request_data.url}"
        )

        return await self.handle_response(
            endpoint=endpoint,
            http_response=HTTPResponse(
                status_code=response.status_code,
                body=response_body,
                headers=response.headers
            )
        )

    async def close_session(self):
        await self.http_session.aclose()



from dataclasses import dataclass
from decimal import Decimal
from sinch.core.models.base_model import SinchBaseModel
from sinch.domains.numbers.enums import NumberType, NumberCapability


@dataclass
class Number(SinchBaseModel):
    phone_number: str
    region_code: str
    type: NumberType
    capability: NumberCapability
    setup_price: dict
    monthly_price: dict
    payment_interval_months: int
    supporting_documentation_required: bool


@dataclass
class ScheduledVoiceProvisioning(SinchBaseModel):
    app_id: str
    status: str
    last_updated_time: str


@dataclass
class VoiceConfiguration(SinchBaseModel):
    app_id: str
    scheduled_provisioning: ScheduledVoiceProvisioning


@dataclass
class SmsConfiguration(SinchBaseModel):
    service_plan_id: str
    scheduled_provisioning: ScheduledVoiceProvisioning


@dataclass
class Money(SinchBaseModel):
    currency_code: str
    amount: Decimal



from enum import Enum


class ConversationProcessingMode(Enum):
    DISPATCH = "DISPATCH"
    CONVERSATION = "CONVERSATION"


class ConversationMetadataReportView(Enum):
    NONE = "NONE"
    FULL = "FULL"


class ConversationRetentionPolicyType(Enum):
    MESSAGE_EXPIRE_POLICY = "MESSAGE_EXPIRE_POLICY"
    CONVERSATION_EXPIRE_POLICY = "CONVERSATION_EXPIRE_POLICY"
    PERSIST_RETENTION_POLICY = "PERSIST_RETENTION_POLICY"


class ConversationChannel(Enum):
    WHATSAPP = "WHATSAPP"
    RCS = "RCS"
    SMS = "SMS"
    MESSENGER = "MESSENGER"
    VIBER = "VIBER"
    VIBERBM = "VIBERBM"
    MMS = "MMS"
    INSTAGRAM = "INSTAGRAM"
    TELEGRAM = "TELEGRAM"
    KAKAOTALK = "KAKAOTALK"
    LINE = "LINE"
    WECHAT = "WECHAT"



import json
from sinch.domains.voice.enums import CalloutMethod
from sinch.core.models.http_response import HTTPResponse
from sinch.domains.voice.endpoints.voice_endpoint import VoiceEndpoint
from sinch.core.enums import HTTPAuthentication, HTTPMethods
from sinch.domains.voice.models.callouts.responses import VoiceCalloutResponse


class CalloutEndpoint(VoiceEndpoint):
    ENDPOINT_URL = "{origin}/calling/v1/callouts"
    HTTP_METHOD = HTTPMethods.POST.value
    HTTP_AUTHENTICATION = HTTPAuthentication.SIGNED.value

    def __init__(self, request_data, callout_method):
        self.request_data = request_data
        self.callout_method = callout_method

    def build_url(self, sinch) -> str:
        return self.ENDPOINT_URL.format(
            origin=sinch.configuration.voice_origin
        )

    def request_body(self):
        request_data = {}
        if self.callout_method == CalloutMethod.TEXT_TO_SPEECH.value:
            request_data["method"] = CalloutMethod.TEXT_TO_SPEECH.value
            request_data[CalloutMethod.TEXT_TO_SPEECH.value] = self.request_data.as_dict()

        elif self.callout_method == CalloutMethod.CUSTOM.value:
            request_data["method"] = CalloutMethod.CUSTOM.value
            request_data[CalloutMethod.CUSTOM.value] = self.request_data.as_dict()

        elif self.callout_method == CalloutMethod.CONFERENCE.value:
            request_data["method"] = CalloutMethod.CONFERENCE.value
            if self.request_data.conferenceDtmfOptions:
                dtmf_options = {}

                if self.request_data.conferenceDtmfOptions["mode"]:
                    dtmf_options["mode"] = self.request_data.get["conferenceDtmfOptions"]["mode"]

                if self.request_data.conferenceDtmfOptions["timeout_mills"]:
                    dtmf_options["timeoutMills"] = self.request_data.get["conferenceDtmfOptions"]["timeout_mills"]

                if self.request_data.conferenceDtmfOptions["max_digits"]:
                    dtmf_options["maxDigits"] = self.request_data.get["conferenceDtmfOptions"]["max_digits"]

                self.request_data.conferenceDtmfOptions = dtmf_options

            request_data[CalloutMethod.CONFERENCE.value] = self.request_data.as_dict()

        return json.dumps(request_data)

    def handle_response(self, response: HTTPResponse):
        super().handle_response(response)
        return VoiceCalloutResponse(call_id=response.body["callId"])



from dataclasses import dataclass
from typing import TypedDict, Literal


@dataclass
class Price:
    currency_id: str
    amount: float


@dataclass
class ConferenceParticipant:
    cli: str
    id: str
    duration: int
    muted: bool
    onhold: bool


@dataclass
class ApplicationNumber:
    number: str
    capability: str
    applicationkey: str


class Destination(TypedDict):
    type: Literal["number", "username"]
    endpoint: str


class ConferenceDTMFOptions(TypedDict):
    mode: Literal["ignore", "forward", "detect"]
    max_digits: int
    timeout_mills: int



from dataclasses import dataclass
from typing import List, Optional
from sinch.core.models.base_model import SinchRequestBaseModel
from sinch.domains.conversation.models import (
    SinchConversationChannelIdentities,
    SinchConversationRecipient
)

from sinch.domains.conversation.enums import (
    ConversationChannel
)


@dataclass
class CreateConversationContactRequest(SinchRequestBaseModel):
    language: str
    channel_identities: Optional[List[SinchConversationChannelIdentities]]
    channel_priority: Optional[List[str]]
    display_name: Optional[str]
    email: Optional[str]
    external_id: Optional[str]
    metadata: Optional[str]


@dataclass
class UpdateConversationContactRequest(CreateConversationContactRequest):
    id: str


@dataclass
class ListConversationContactRequest(SinchRequestBaseModel):
    page_size: int
    page_token: str
    external_id: str
    channel: str
    identity: str


@dataclass
class DeleteConversationContactRequest(SinchRequestBaseModel):
    contact_id: str


@dataclass
class GetConversationContactRequest(SinchRequestBaseModel):
    contact_id: str


@dataclass
class MergeConversationContactsRequest(SinchRequestBaseModel):
    destination_id: str
    source_id: str
    strategy: str


@dataclass
class GetConversationChannelProfileRequest(SinchRequestBaseModel):
    app_id: str
    recipient: SinchConversationRecipient
    channel: ConversationChannel



from abc import ABC, abstractmethod


class PageIterator:
    def __init__(self, paginator):
        self.paginator = paginator

    def __iter__(self):
        return self

    def __next__(self):
        if self.paginator.has_next_page:
            return self.paginator.next_page()
        else:
            raise StopIteration


class AsyncPageIterator:
    def __init__(self, paginator):
        self.paginator = paginator

    def __aiter__(self):
        return self

    async def __anext__(self):
        if self.paginator.has_next_page:
            return await self.paginator.next_page()
        else:
            raise StopAsyncIteration


class Paginator(ABC):
    """
    Pagination response object.

    auto_paging_iter method returns an iterator object that can be used for iterator-based page traversing.
    For example:
        for page in paginated_response.auto_paging_iter():
            ...process page object

    For manual pagination use has_next_page property with next_page() method.
    For example:
        if paginated_response.has_next_page:
            paginated_response = paginated_response.next_page()
    """
    def __init__(self, sinch, endpoint, result):
        self._sinch = sinch
        self.result = result
        self.endpoint = endpoint
        self.has_next_page = False
        self._calculate_next_page()

    def __repr__(self):
        return "Paginated response content: " + str(self.result)

    @abstractmethod
    def auto_paging_iter(self):
        pass

    @abstractmethod
    def next_page(self):
        pass

    @abstractmethod
    def _calculate_next_page(self):
        pass

    @classmethod
    @abstractmethod
    def _initialize(cls, sinch, endpoint):
        pass


class IntBasedPaginator(Paginator):
    __doc__ = Paginator.__doc__

    def _calculate_next_page(self):
        if self.result.page_size:
            self.has_next_page = True
        else:
            self.has_next_page = False

    def next_page(self):
        self.endpoint.request_data.page += 1
        self.result = self._sinch.configuration.transport.request(self.endpoint)
        self._calculate_next_page()
        return self

    def auto_paging_iter(self):
        return PageIterator(self)

    @classmethod
    def _initialize(cls, sinch, endpoint):
        result = sinch.configuration.transport.request(endpoint)
        return cls(sinch, endpoint, result)


class AsyncIntBasedPaginator(IntBasedPaginator):
    __doc__ = IntBasedPaginator.__doc__

    async def next_page(self):
        self.endpoint.request_data.page += 1
        self.result = await self._sinch.configuration.transport.request(self.endpoint)
        self._calculate_next_page()
        return self

    def auto_paging_iter(self):
        return AsyncPageIterator(self)

    @classmethod
    async def _initialize(cls, sinch, endpoint):
        result = await sinch.configuration.transport.request(endpoint)
        return cls(sinch, endpoint, result)


class TokenBasedPaginator(Paginator):
    __doc__ = Paginator.__doc__

    def _calculate_next_page(self):
        if self.result.next_page_token:
            self.has_next_page = True
        else:
            self.has_next_page = False

    def next_page(self):
        self.endpoint.request_data.page_token = self.result.next_page_token
        self.result = self._sinch.configuration.transport.request(self.endpoint)
        self._calculate_next_page()
        return self

    def auto_paging_iter(self):
        return PageIterator(self)

    @classmethod
    def _initialize(cls, sinch, endpoint):
        result = sinch.configuration.transport.request(endpoint)
        return cls(sinch, endpoint, result)


class AsyncTokenBasedPaginator(TokenBasedPaginator):
    __doc__ = TokenBasedPaginator.__doc__

    async def next_page(self):
        self.endpoint.request_data.page_token = self.result.next_page_token
        self.result = await self._sinch.configuration.transport.request(self.endpoint)
        self._calculate_next_page()
        return self

    def auto_paging_iter(self):
        return AsyncPageIterator(self)

    @classmethod
    async def _initialize(cls, sinch, endpoint):
        result = await sinch.configuration.transport.request(endpoint)
        return cls(sinch, endpoint, result)



from dataclasses import dataclass
from typing import List
from sinch.core.models.base_model import SinchBaseModel
from sinch.domains.sms.models.inbounds import InboundMessage


@dataclass
class SinchListInboundMessagesResponse(SinchBaseModel):
    page: str
    page_size: str
    count: str
    inbounds: List[InboundMessage]


@dataclass
class GetInboundMessagesResponse(InboundMessage):
    pass



from dataclasses import dataclass

from sinch.core.models.base_model import SinchBaseModel


@dataclass
class Conversation(SinchBaseModel):
    id: str
    app_id: str
    contact_id: str
    last_received: str
    active_channel: str
    active: str
    metadata: str
    metadata_json: str



from sinch.core.models.http_response import HTTPResponse
from sinch.domains.conversation.endpoints.conversation_endpoint import ConversationEndpoint
from sinch.core.enums import HTTPAuthentication, HTTPMethods
from sinch.domains.conversation.models.conversation.responses import SinchCreateConversationResponse
from sinch.domains.conversation.models.conversation.requests import CreateConversationRequest


class CreateConversationEndpoint(ConversationEndpoint):
    ENDPOINT_URL = "{origin}/v1/projects/{project_id}/conversations"
    HTTP_METHOD = HTTPMethods.POST.value
    HTTP_AUTHENTICATION = HTTPAuthentication.OAUTH.value

    def __init__(self, project_id: str, request_data: CreateConversationRequest):
        super(CreateConversationEndpoint, self).__init__(project_id, request_data)
        self.request_data = request_data
        self.project_id = project_id

    def build_url(self, sinch):
        return self.ENDPOINT_URL.format(
            origin=sinch.configuration.conversation_origin,
            project_id=self.project_id
        )

    def request_body(self):
        return self.request_data.as_json()

    def handle_response(self, response: HTTPResponse) -> SinchCreateConversationResponse:
        super(CreateConversationEndpoint, self).handle_response(response)
        return SinchCreateConversationResponse(
            id=response.body["id"],
            app_id=response.body["app_id"],
            contact_id=response.body["contact_id"],
            last_received=response.body["last_received"],
            active_channel=response.body["active_channel"],
            active=response.body["active"],
            metadata=response.body["metadata"],
            metadata_json=response.body["metadata_json"]
        )



from dataclasses import dataclass
from typing import List

from sinch.core.models.base_model import SinchBaseModel
from sinch.domains.conversation.models.templates import ConversationTemplate


@dataclass
class CreateConversationTemplateResponse(ConversationTemplate):
    pass


@dataclass
class ListConversationTemplatesResponse(SinchBaseModel):
    templates: List[ConversationTemplate]


@dataclass
class GetConversationTemplateResponse(ConversationTemplate):
    pass


@dataclass
class DeleteConversationTemplateResponse(SinchBaseModel):
    pass


@dataclass
class UpdateConversationTemplateResponse(ConversationTemplate):
    pass



from sinch.core.deserializers import timestamp_to_datetime_in_utc_deserializer
from sinch.core.models.http_response import HTTPResponse
from sinch.domains.voice.endpoints.voice_endpoint import VoiceEndpoint
from sinch.core.enums import HTTPAuthentication, HTTPMethods
from sinch.domains.voice.models.calls.responses import GetVoiceCallResponse
from sinch.domains.voice.models.calls.requests import GetVoiceCallRequest
from sinch.domains.voice.models import Price, Destination


class GetCallEndpoint(VoiceEndpoint):
    ENDPOINT_URL = "{origin}/calling/v1/calls/id/{call_id}"
    HTTP_METHOD = HTTPMethods.GET.value
    HTTP_AUTHENTICATION = HTTPAuthentication.SIGNED.value

    def __init__(self, request_data: GetVoiceCallRequest):
        self.request_data = request_data

    def build_url(self, sinch) -> str:
        return self.ENDPOINT_URL.format(
            origin=sinch.configuration.voice_origin,
            call_id=self.request_data.call_id
        )

    def handle_response(self, response: HTTPResponse) -> GetVoiceCallResponse:
        super().handle_response(response)
        call_origin = response.body.get("from")
        call_destination = response.body.get("to")
        return GetVoiceCallResponse(
            from_=Destination(
                type=call_origin["type"],
                endpoint=call_origin.get["endpoint"],
            ) if call_origin else None,
            to=Destination(
                type=call_destination.get("type"),
                endpoint=call_destination.get("endpoint")
            ) if call_destination else None,
            domain=response.body.get("domain"),
            call_id=response.body.get("callId"),
            duration=response.body.get("duration"),
            status=response.body.get("status"),
            result=response.body.get("result"),
            reason=response.body.get("reason"),
            timestamp=timestamp_to_datetime_in_utc_deserializer(response.body["timestamp"]),
            custom=response.body.get("custom"),
            user_rate=Price(
                currency_id=response.body["userRate"]["currencyId"],
                amount=response.body["userRate"]["amount"]
            ),
            debit=Price(
                currency_id=response.body["userRate"]["currencyId"],
                amount=response.body["userRate"]["amount"]
            )
        )



from dataclasses import dataclass
from sinch.core.models.base_model import SinchBaseModel


@dataclass
class QueryConversationCapabilityResponse(SinchBaseModel):
    request_id: str
    app_id: str
    recipient: dict



from typing import List
from dataclasses import dataclass
from sinch.core.models.base_model import SinchRequestBaseModel


@dataclass
class AssignNumbersVoiceApplicationRequest(SinchRequestBaseModel):
    numbers: List[str]
    application_key: str
    capability: str


@dataclass
class UnassignNumbersVoiceApplicationRequest(SinchRequestBaseModel):
    number: str
    application_key: str
    capability: str


@dataclass
class QueryNumberVoiceApplicationRequest(SinchRequestBaseModel):
    number: str


@dataclass
class UpdateCallbackUrlsVoiceApplicationRequest(SinchRequestBaseModel):
    application_key: str
    primary: str
    fallback: str


@dataclass
class GetCallbackUrlsVoiceApplicationRequest(SinchRequestBaseModel):
    application_key: str



class SinchException(Exception):
    def __init__(self, message, response, is_from_server):
        self.is_from_server = is_from_server
        self.response_status_code = response.status_code if response else None
        self.http_response = response
        super().__init__(message)


class ValidationException(SinchException):
    pass



from dataclasses import dataclass

from sinch.core.models.base_model import SinchRequestBaseModel


@dataclass
class ListAvailableRegionsForProjectRequest(SinchRequestBaseModel):
    number_type: str
    number_types: list



from sinch.core.models.http_response import HTTPResponse
from sinch.core.endpoint import HTTPEndpoint
from sinch.domains.sms.exceptions import SMSException
from sinch.core.enums import HTTPAuthentication


class SMSEndpoint(HTTPEndpoint):
    def __init__(self, request_data, sinch):
        self.request_data = request_data
        self.sinch = sinch

        if sinch.configuration.service_plan_id:
            self.project_or_service_id = sinch.configuration.service_plan_id
            self.HTTP_AUTHENTICATION = HTTPAuthentication.SMS_TOKEN.value
            self.sms_origin = self.sinch.configuration.sms_origin_with_service_plan_id

        else:
            self.project_or_service_id = sinch.configuration.project_id
            self.sms_origin = self.sinch.configuration.sms_origin

    def handle_response(self, response: HTTPResponse):
        if response.status_code >= 400:
            raise SMSException(
                message=response.body["text"],
                response=response,
                is_from_server=True
            )
